<?php 
/*
      +----------------------------------------------------------------------+
      | Copyright (c) 2001, 2003 Universite catholique de Louvain (UCL)      |
      | Copyright (c) 2005 	 Pierre Gorissen			     |
      +----------------------------------------------------------------------+
      |   This program is free software; you can redistribute it and/or      |
      |   modify it under the terms of the GNU General Public License        |
      |   as published by the Free Software Foundation; either version 2     |
      |   of the License, or (at your option) any later version.             |
      +----------------------------------------------------------------------+
      | Authors: Pierre Gorissen <Pierre@gosoftonline.com>	             |
      | Authors: Olivier Brouckaert <oli.brouckaert@skynet.be>               |
      +----------------------------------------------------------------------+
*/

require_once("pclzip/pclzip.lib.php");
include_once("class_xslt/class_xslt.php");

function unique_id()
{
	return md5(uniqid(rand(), true));

}

if(!class_exists('QTIQuestion')):

		/*>>>>>>>>>>>>>>>>>>>> CLASS QTIQUESTION <<<<<<<<<<<<<<<<<<<<*/

/**
 * This class allows to instantiate an object of type QTIQuestion
 */
class QTIQuestion extends Question
{
	var $id;
	var $question;
	var $description;
	var $weighting;
	var $position;
	var $type;
	var $picture;
	
	var $exerciseList;  // array with the list of exercises which this question is in

	/**
	 * constructor of the class
	 *
	 * @author - Pierre Gorissen
	 */
	function QTIQuestion()
	{
		$this->id=0;
		$this->item='';
		$this->question='';
		$this->description='';
		$this->weighting=0;
		$this->position=1;
		$this->type=2;
		$this->picture='';
	}

	/**
 	* display a question and its answers
 	*
 	* @displays 'number of answers' if question exists, otherwise false
 	*
 	* @author Pierre Gorissen <Pierre@gosoftonline.com>
 	 	*
 	* @param integer	$questionId	ID of the question to show
 	*/
	function displayQTIQuestion($questionId)
	{


		echo htmlentities($this->buildQTIQuestion($questionId));



	}

	/**
 	* export QTI question and its answers
 	*
 	* @creates a zip-file with selected question
 	*
 	* @author Pierre Gorissen <Pierre@gosoftonline.com>
 	*
 	* @param integer	$questionId	ID of the question to export
 	*/
	function exportQTIQuestion($questionId)
	{

		global $rootSys;

		$basePath = $rootSys."claroline/exercice/ims";
		$xsdPath  = $basePath."/xsd";
		$item_id = unique_id(); // unique id for this item
		$tmpPath  = $basePath."/tmp/".$item_id;
		mkdir($tmpPath);
		$xsltPath = $basePath."/xslt";

  
		// create empty zipfile
		$zipfilename = $tmpPath."/export.zip";
		$zipfile = new PclZip($zipfilename);
  	
		// add default schema files to zipfile
		$v_list = $zipfile->add($xsdPath."/imsqti_v2p0.xsd", PCLZIP_OPT_REMOVE_PATH, $xsdPath); 
		$v_list = $zipfile->add($xsdPath."/imscp_v1p1.xsd", PCLZIP_OPT_REMOVE_PATH, $xsdPath);
		$v_list = $zipfile->add($xsdPath."/ims_xml.xsd", PCLZIP_OPT_REMOVE_PATH, $xsdPath);  
		$v_list = $zipfile->add($xsdPath."/imsmd_v1p2p2.xsd", PCLZIP_OPT_REMOVE_PATH, $xsdPath);  


		// generate and save QTI item

		$item = $this->buildQTIQuestion($questionId,$item_id);
		$QTIfilename = $tmpPath."/question_".$questionId.".xml";
		$handle = fopen ($QTIfilename, "w");
		fwrite($handle, $item);
		fclose($handle);
		$v_list = $zipfile->add($QTIfilename, PCLZIP_OPT_REMOVE_PATH, $tmpPath);  
		unlink($QTIfilename);


		// generate resource metadata
		$xslt=new Xslt();
		$xslt->setXmlString($item);
		$xslt->setXsl($xsltPath."/QTI2IMSMD.xslt");
		if($xslt->transform()) 
		{
   			$resource_metadata=$xslt->getOutput();
			$resource_metadata=preg_replace("/<\?xml(.*?)>/i","", $resource_metadata);
			$resource_metadata=preg_replace("/<!--\s(.*?)-->/s","", $resource_metadata);
			$resource_metadata=trim($resource_metadata);
		} 

		// generate item profile information
		$xslt->setXmlString($item);
		$xslt->setXsl($xsltPath."/QTIv2itemProfile.xslt");
		if($xslt->transform()) 
		{
   			$profile=$xslt->getOutput();

			$Profilefilename = $tmpPath."/profile_question_".$questionId.".xml";
			$handle = fopen ($Profilefilename, "w");
			fwrite($handle, $profile);
			fclose($handle);
			$v_list = $zipfile->add($Profilefilename, PCLZIP_OPT_REMOVE_PATH, $tmpPath);
			unlink($Profilefilename);
		} 

		$manifest_Template = $this->readTemplatefile("imsmanifest");
		$manifest_item_MD_Template = $this->readTemplatefile("imsmanifest_item_MD");
		$manifest_profile_MD_Template = $this->readTemplatefile("imsmanifest_profile_MD");
		$manifest_resources_Template = $this->readTemplatefile("imsmanifest_resources");

		$MDTemplateValues["ITEMID"] = "question_".$questionId;
		$MDTemplateValues["PROFILEID"] = "profile_question_".$questionId;


		$itemMD = $this->fillTemplate($manifest_item_MD_Template,$MDTemplateValues); 
		$profileMD = $this->fillTemplate($manifest_profile_MD_Template,$MDTemplateValues); 

		$manifestTemplateValues["ITEMFILENAME"] = "question_".$questionId.".xml";
		$manifestTemplateValues["PROFILEFILENAME"] = "profile_question_".$questionId.".xml";
		$manifestTemplateValues["RESOURCEMETADATA"] = $resource_metadata;
		$manifestTemplateValues["QTIFILEMETADATA"] = $itemMD;
		$manifestTemplateValues["PROFILEFILEMETADATA"] = $profileMD;
		$manifestTemplateValues["ITEMID"] = "question_".$questionId."_".$item_id;
		$manifestTemplateValues["MANIFESTID"] = "manifest_".$item_id;

		$manifestTemplateValues["RESOURCES"] = $this->fillTemplate($manifest_resources_Template,$manifestTemplateValues); 

		$manifest = $this->fillTemplate($manifest_Template,$manifestTemplateValues); 

		$Manifestfilename = $tmpPath."/imsmanifest.xml";
		$handle = fopen ($Manifestfilename, "w");
		fwrite($handle, $manifest);
		fclose($handle);
		$v_list = $zipfile->add($Manifestfilename, PCLZIP_OPT_REMOVE_PATH, $tmpPath);  
		unlink($Manifestfilename);

		header("Pragma: public"); // required
		header("Expires: 0");
		header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
		header("Cache-Control: private",false); // required for certain browsers 
		header("Content-Type: application/octet-stream");
		header("Content-Disposition: attachment; filename=".basename($zipfilename).";" );
		header("Content-Transfer-Encoding: binary");
		header("Content-Length: ".filesize($zipfilename));
		readfile("$zipfilename");

		unlink($zipfilename);	
		rmdir($tmpPath);

	}



	/**
 	* builds a question and its answers
 	*
 	* @returns QTI version of question if question exists, otherwise false
 	*
 	* @author Pierre Gorissen <Pierre@gosoftonline.com>
 	* @based on the EXERCISE TOOL LIBRARY 
 	*
 	* @param integer	$questionId	ID of the question to show
 	*/
	function buildQTIQuestion($questionId,$item_id)
	{
		// array to hold the template values
        	$itemTemplateValues=array();
		$answerTemplateValues=array();

		// reset all temporary values;
		$item = "";
		$item_maps = "";
		$item_simple_choices = "";
		$item_correct_responses = "";
		$item_textentry_interactions = "";
		$item_match_set1 = "";
		$item_match_set2 = "";
		$match_maxmatch2 = 0;

		// reads question informations
		if(!$this->read($questionId))
		{
			// question not found
			return false;
		}

		$answerType=$this->selectType();
		// echo "answerType = ".$answerType."<br />";

		$itemTemplateValues["QUESTION_ID"]  = $this->selectId();
		$itemTemplateValues["QUESTION_ID"] .= "_".$item_id;
		$itemTemplateValues["QUESTION"] = $this->selectTitle();
		$itemTemplateValues["QUESTIONPONDERATION"] = $this->selectWeighting();
		$itemTemplateValues["QUESTIONDESCRIPTION"] = $this->selectDescription();


		$itemTemplateValues["CARDINALITY"] = "multiple";
		if ($answerType == UNIQUE_ANSWER) $itemTemplateValues["CARDINALITY"] = "single";
		if ($answerType == MULTIPLE_ANSWER) $itemTemplateValues["CARDINALITY"] = "multiple";
		if ($answerType == FILL_IN_BLANKS) $itemTemplateValues["CARDINALITY"] = "single";

		// unique answer or multiple answer
		if($answerType == UNIQUE_ANSWER || $answerType == MULTIPLE_ANSWER)
		{
			// load templates
			$choice_item_Template = $this->readTemplatefile("choice_item");
			$choice_maps_Template = $this->readTemplatefile("item_maps");
			$choice_simple_choices_Template = $this->readTemplatefile("choice_item_simple_choices");
			$correct_response_Template = $this->readTemplatefile("correct_response");

			$item = $this->fillTemplate($choice_item_Template,$itemTemplateValues); 

		}

		// fill in blanks
		elseif($answerType == FILL_IN_BLANKS)	
		{
			// load templates
			$fib_item_Template = $this->readTemplatefile("fib_item");
			$fib_response_processing_Template = $this->readTemplatefile("fib_response_processing");
			$fib_response_declaration_Template = $this->readTemplatefile("fib_response_declaration");

			$item = $this->fillTemplate($fib_item_Template,$itemTemplateValues); 

		}
		// matching
		else
		{
			// load templates
			$match_item_Template = $this->readTemplatefile("match_item");

			$item = $this->fillTemplate($match_item_Template,$itemTemplateValues); 

		}

		// construction of the Answer object
		$objAnswerTmp=new Answer($questionId);

		// get number of answers for question
		$nbrAnswers=$objAnswerTmp->selectNbrAnswers();

		// process answers for question
		for($answerId=1;$answerId <= $nbrAnswers;$answerId++)
		{

			$answerCorrect=$objAnswerTmp->isCorrect($answerId);

			// unique answer or multiple answers
			if($answerType == UNIQUE_ANSWER || $answerType == MULTIPLE_ANSWER)
			{
				// save answer information in array
				$answerTemplateValues["ANSWERID"] = $answerId;
				$answerTemplateValues["ANSWERPONDERATION"] = $objAnswerTmp->selectWeighting($answerId);
				$answerTemplateValues["ANSWEROPTION"] = $objAnswerTmp->selectAnswer($answerId);
				$answerTemplateValues["ANSWERCOMMENT"] = $objAnswerTmp->selectComment($answerId);


				if($answerCorrect)
				{
					$answerTemplateValues["CORRECTRESPONSE"] = $answerId;
					$item_correct_responses .= $this->fillTemplate($correct_response_Template,$answerTemplateValues) ;
				}


				$item_maps .= $this->fillTemplate($choice_maps_Template,$answerTemplateValues) ;
				$item_simple_choices .= $this->fillTemplate($choice_simple_choices_Template, $answerTemplateValues) ;



			}
			// fill in blanks
			elseif($answerType == FILL_IN_BLANKS)	
			{
				// save answer information in array
				$answer = $objAnswerTmp->selectAnswer($answerId);

				// splits text and weightings that are joined with the character '::'
				list($answer, $weights)=explode('::',$answer);
				$weights=explode(',',$weights);


				// because [] is parsed here we follow this procedure:
				// 1. find everything between the [tex] and [/tex] tags
				$startlocations=strpos($answer,'[tex]');
				$endlocations=strpos($answer,'[/tex]');

				if($startlocations !== false && $endlocations !== false)
				{
					$texstring=substr($answer,$startlocations,$endlocations-$startlocations+6);
					// 2. replace this by {texcode}
					$answer=str_replace($texstring,'{texcode}',$answer);
				}



				// 3. do the normal matching parsing
				$fibint = 1;
				foreach ($weights as $answerweight)
				{
					// find single [blank]
					preg_match("/\[[^]]+\]/i",$answer, $matches);
					$fibfield = $matches[0];
					$fibkeyvalue = str_replace("]","", str_replace("[","", $fibfield));
			
					// replaces [blank] with textEntryInteractin
				   	$answer=str_replace($fibfield,'<textEntryInteraction responseIdentifier="RESPONSE'.$fibint.'" expectedLength="10" />', nl2br($answer));

					// save answer information in array
					$answerTemplateValues["ANSWERID"] = $fibint;
					$answerTemplateValues["RESPONSE_ID"] = $answerTemplateValues["ANSWERID"];
					$answerTemplateValues["ANSWERPONDERATION"] = $answerweight;
					$answerTemplateValues["ANSWER"] = $fibkeyvalue;
					$answerTemplateValues["CARDINALITY"] = "single";

					$item_responsedeclarations .=$this->fillTemplate($fib_response_declaration_Template,$answerTemplateValues) ;
					$item_responseprocessing .=$this->fillTemplate($fib_response_processing_Template,$answerTemplateValues) ;

					$fibint++;

				}
				// 4. replace the {texcode by the api_pare_tex parsed code}
				$texstring = api_parse_tex($texstring);
				$answer=str_replace("{texcode}",$texstring,$answer);

				$item_textentryinteractions=$answer;
				


			}
			// matching
			else
			{
				// save answer information in array

				$answer = $objAnswerTmp->selectAnswer($answerId);

				$answerWeight = $objAnswerTmp->selectWeighting($answerId);
				$correctAnswer = "M".$answerId." M".$answerCorrect." ";

				if($answerCorrect)
				{

					$item_maps .= "<mapEntry mapKey=\"".$correctAnswer."\" mappedValue=\"".$answerWeight."\"/>\n";
					$item_correct_responses .= "<value>".$correctAnswer."</value>\n";
					$item_match_set1 .= "<simpleAssociableChoice identifier=\"M".$answerId.
						"\" matchMax=\"{QTI_MAXMATCH1}\">".$answer.
						"</simpleAssociableChoice>\n";

					$match_maxmatch2++;
				}
				else
				{
					$item_match_set2 .= "<simpleAssociableChoice identifier=\"M".$answerId.
						"\" matchMax=\"{QTI_MAXMATCH2}\">".$answer.
						"</simpleAssociableChoice>\n";
	
				}

				


			}
		}	// end for()

		// destruction of the Answer object
		unset($objAnswerTmp);

		// multiple choice or answers

		if($answerType == UNIQUE_ANSWER || $answerType == MULTIPLE_ANSWER)
		{
			// complete templates
			$itemTemplateValues["MAPS"] = $item_maps;
			$itemTemplateValues["SIMPLECHOICES"] = $item_simple_choices;
			$itemTemplateValues["CORRECTRESPONSES"] = $item_correct_responses;
			if($answerType == UNIQUE_ANSWER) $itemTemplateValues["MAXCHOICES"] = 1;
			if($answerType == MULTIPLE_ANSWER) $itemTemplateValues["MAXCHOICES"] = 0;	

			$item = $this->fillTemplate($item,$itemTemplateValues); 
		}
		// fill in blanks
		elseif($answerType == FILL_IN_BLANKS)	
		{
			// complete templates
			$itemTemplateValues["TEXTENTRYINTERACTIONS"] = $item_textentryinteractions;
			$itemTemplateValues["RESPONSEDECLARATIONS"] = $item_responsedeclarations;
			$itemTemplateValues["RESPONSEPROCESSING"] = $item_responseprocessing;
			$itemTemplateValues["MAXCHOICES"] = 1;

			$item = $this->fillTemplate($item,$itemTemplateValues); 

		}
		// matching
		else
		{
			// complete templates
			$itemTemplateValues["CORRECTRESPONSES"] = $item_correct_responses;
			$itemTemplateValues["MAPS"] = $item_maps;
			$itemTemplateValues["SIMPLEMATCHSET1"] = $item_match_set1;
			$itemTemplateValues["SIMPLEMATCHSET2"] = $item_match_set2;
			$itemTemplateValues["MAXMATCH1"] = 1;
			$itemTemplateValues["MAXMATCH2"] = $match_maxmatch2;
			$itemTemplateValues["MAXCHOICES"] = $match_maxmatch2;

			$item = $this->fillTemplate($item,$itemTemplateValues); 

		}


		return $item;
	}



	/**
	 * reads the template file from file
	 * based on the template type
	 *
 	 * @author Pierre Gorissen <Pierre@gosoftonline.com>
	 *
	 * @templateType  type of template to load
	 *
	 */
	function readTemplatefile($templateType)
	{

		global $rootSys;

		// template path
		$basePath 	= $rootSys."claroline/exercice/ims";
		$templatePath	= $basePath."/template";

		$templateNames = array();

		$templateNames["choice_item"]                	=  $templatePath."/choice_item_template.xml" ;
		$templateNames["item_maps"]           	     	=  $templatePath."/item_maps_template.xml" ;
		$templateNames["choice_item_simple_choices"] 	=  $templatePath."/choice_item_simple_choices_template.xml" ;
		$templateNames["correct_response"]  		=  $templatePath."/correct_response_template.xml";
		$templateNames["fib_item"]                   	=  $templatePath."/fib_item_template.xml" ;
		$templateNames["fib_response_processing"]       =  $templatePath."/fib_response_processing_template.xml" ;
		$templateNames["fib_response_declaration"]      =  $templatePath."/fib_response_declaration_template.xml" ;
		$templateNames["match_item"]                   	=  $templatePath."/match_item_template.xml" ;
		$templateNames["imsmanifest"]                  	=  $templatePath."/imsmanifest_template.xml";
		$templateNames["imsmanifest_item_MD"]           =  $templatePath."/imsmanifest_item_MD_template.xml";
		$templateNames["imsmanifest_profile_MD"]        =  $templatePath."/imsmanifest_profile_MD_template.xml";
		$templateNames["imsmanifest_resources"]         =  $templatePath."/imsmanifest_resources_template.xml";


		$templateName = $templateNames[$templateType];


		$handle = @fopen ($templateName, "r"); 
		$filedata = @fread ($handle, filesize ($templateName)); 
		fclose ($handle);
		

		return $filedata;	


	}


	/**
	  * fills a template with values
	  * (static)
	  *
 	  * @author Pierre Gorissen <Pierre@gosoftonline.com>
	  *
	  * @param $template 
	  *		Template to be used
	  * @param $values
	  *		Array of all the values 
	  */
	function fillTemplate($template, $values) {

		if (sizeof($values) != 0) {

			// go through all the values
			foreach ($values as $key => $value) 
			{ 		
				$template = str_replace("{QTI_".$key."}",$values[$key],$template);
			}
		}

		return $template;

	}

}

endif;
?>