<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:imsmd="http://www.imsglobal.org/xsd/imsmd_v1p2" xmlns:imsqti="http://www.imsglobal.org/xsd/imsqti_v2p0" >
	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>
	<!--
	(c) 2004 SURF SiX http://e-learning.surf.nl/SiX/english
	Author: Pierre Gorissen - P.Gorissen@Fontys.nl
	This XSL transform extract the minimal metadata from a QTI item to include in an IMS CP
	All reuse permitted provided this notice remains intact
	-->
	<!-- TODO
	* check for language in item
	* ....
	
	-->
	<!-- VERSION HISTORY
	Version 0.1 - first version, works with QTI ITEM V2.0 draft
	Version 0.2 - updated for changes in QTI V2.0 final
	Version 0.3 - some bugfixes 
	-->
	<xsl:template match="/">
		<xsl:comment>
		Converted by the QTI2IMSMD XLST script version 0.3
		Cut and paste the metadata element into an imsmanifest file for the assessment item.
		</xsl:comment>
		<metadata>
			<schema>IMS QTI</schema>
			<schemaversion>2.0</schemaversion>
			<imsmd:lom>
				<imsmd:general>
					<imsmd:identifier>
						<xsl:value-of select="/imsqti:assessmentItem/attribute::identifier"/>
					</imsmd:identifier>
					<imsmd:title>
						<imsmd:langstring xml:lang="en">
							<xsl:value-of select="//imsqti:assessmentItem/attribute::title"/>
						</imsmd:langstring>
					</imsmd:title>
					<imsmd:description>
						<imsmd:langstring xml:lang="en">
							<xsl:value-of select="//imsqti:assessmentItem/attribute::title"/>
						</imsmd:langstring>
					</imsmd:description>
				</imsmd:general>
				<imsmd:metametadata>
					<imsmd:identifier><xsl:value-of select="/imsqti:assessmentItem/attribute::identifier"/>_MD</imsmd:identifier>
					<imsmd:metadatascheme>LOMv1.0</imsmd:metadatascheme>
					<imsmd:metadatascheme>IMSQTIv2.0</imsmd:metadatascheme>
					<imsmd:language>en</imsmd:language>
				</imsmd:metametadata>
				<imsmd:technical>
					<imsmd:format>text/x-imsqti-item-xml</imsmd:format>
				</imsmd:technical>
				<imsmd:educational>
					<imsmd:learningresourcetype>
						<imsmd:source>
							<imsmd:langstring xml:lang="en">RDNLTSNType</imsmd:langstring>
						</imsmd:source>
						<imsmd:value>
							<imsmd:langstring xml:lang="x-none">AssessmentItem</imsmd:langstring>
						</imsmd:value>
					</imsmd:learningresourcetype>
				</imsmd:educational>
			</imsmd:lom>
			<imsqti:qtiMetadata>
				<xsl:call-template name="itemTemplate"/>
				<xsl:apply-templates select="/imsqti:assessmentItem" mode="timeDependent"/>
				<xsl:call-template name="compositeItem"/>
				<xsl:apply-templates select="/imsqti:assessmentItem" mode="interactionType"/>
				<xsl:call-template name="feedbackType"/>
				<xsl:call-template name="solutionAvailable"/>
			</imsqti:qtiMetadata>
		</metadata>
	</xsl:template>
	<xsl:template match="imsqti:assessmentItem" mode="timeDependent">
		<imsqti:timeDependent>
			<xsl:value-of select="attribute::timeDependent"/>
		</imsqti:timeDependent>
	</xsl:template>
	<xsl:template match="imsqti:assessmentItem" mode="interactionType">
		<xsl:apply-templates select="//imsqti:associateInteraction[1]
				| //imsqti:choiceInteraction[1]
				| //imsqti:choiceMatrixInteraction[1]
				| //imsqti:customInteraction[1]
				| //imsqti:drawingInteraction[1]
				| //imsqti:endAttemptInteraction[1]
				| //imsqti:extendedTextInteraction[1]
				| //imsqti:gapMatchInteraction[1]
				| //imsqti:graphicAssociateInteraction[1]
				| //imsqti:graphicGapMatchInteraction[1]
				| //imsqti:graphicOrderInteraction[1]
				| //imsqti:hotspotInteraction[1]
				| //imsqti:hottextInteraction[1]
				| //imsqti:matchInteraction[1]
				| //imsqti:orderInteraction[1]
				| //imsqti:positionObjectInteraction[1]
				| //imsqti:selectPointInteraction[1]
				| //imsqti:sliderInteraction[1]
				| //imsqti:inlineChoiceInteraction[1]
				| //imsqti:textEntryInteraction[1]
				| //imsqti:uploadInteraction[1]" mode="interactionType"/>
	</xsl:template>
	<xsl:template match="//imsqti:associateInteraction
				| //imsqti:choiceInteraction
				| //imsqti:choiceMatrixInteraction
				| //imsqti:customInteraction
				| //imsqti:drawingInteraction
				| //imsqti:endAttemptInteraction
				| //imsqti:extendedTextInteraction
				| //imsqti:gapMatchInteraction
				| //imsqti:graphicAssociateInteraction
				| //imsqti:graphicGapMatchInteraction
				| //imsqti:graphicOrderInteraction
				| //imsqti:hotspotInteraction
				| //imsqti:hottextInteraction
				| //imsqti:matchInteraction
				| //imsqti:orderInteraction
				| //imsqti:positionObjectInteraction
				| //imsqti:selectPointInteraction
				| //imsqti:sliderInteraction
				| //imsqti:inlineChoiceInteraction
				| //imsqti:textEntryInteraction
				| //imsqti:uploadInteraction" mode="interactionType">
		<imsqti:interactionType>
			<xsl:value-of select="name()"/>
		</imsqti:interactionType>
	</xsl:template>
	<xsl:template name="itemTemplate">
		<xsl:param name="tlist" select="//imsqti:templateProcessing"/>
		<xsl:param name="count">1</xsl:param>
		<imsqti:itemTemplate>
			<xsl:choose>
				<xsl:when test="$count &lt;= count($tlist)">
					<xsl:text>true</xsl:text>
				</xsl:when>
				<xsl:otherwise>
					<xsl:text>false</xsl:text>
				</xsl:otherwise>
			</xsl:choose>
		</imsqti:itemTemplate>
	</xsl:template>
	<xsl:template name="compositeItem">
		<xsl:param name="ilist" select="//imsqti:associateInteraction
				| //imsqti:choiceInteraction
				| //imsqti:choiceMatrixInteraction
				| //imsqti:customInteraction
				| //imsqti:drawingInteraction
				| //imsqti:endAttemptInteraction
				| //imsqti:extendedTextInteraction
				| //imsqti:gapMatchInteraction
				| //imsqti:graphicAssociateInteraction
				| //imsqti:graphicGapMatchInteraction
				| //imsqti:graphicOrderInteraction
				| //imsqti:hotspotInteraction
				| //imsqti:hottextInteraction
				| //imsqti:matchInteraction
				| //imsqti:orderInteraction
				| //imsqti:positionObjectInteraction
				| //imsqti:selectPointInteraction
				| //imsqti:sliderInteraction
				| //imsqti:inlineChoiceInteraction
				| //imsqti:textEntryInteraction
				| //imsqti:uploadInteraction"/>
		<xsl:param name="count">1</xsl:param>
		<imsqti:composite>
			<xsl:choose>
				<xsl:when test="$count &lt; count($ilist)">
					<xsl:text>true</xsl:text>
				</xsl:when>
				<xsl:otherwise>
					<xsl:text>false</xsl:text>
				</xsl:otherwise>
			</xsl:choose>
		</imsqti:composite>
	</xsl:template>
	<xsl:template name="feedbackType">
		<xsl:param name="adaptive" select="/imsqti:assessmentItem/attribute::adaptive"/>
		<xsl:param name="fblist" select="//imsqti:modalFeedback | //imsqti:feedbackInline | //imsqti:feedbackBlock"/>
		<xsl:param name="count">1</xsl:param>
		<xsl:param name="feedback">
			<xsl:choose>
				<xsl:when test="$count &lt;= count($fblist)">
					<xsl:text>true</xsl:text>
				</xsl:when>
				<xsl:otherwise>
					<xsl:text>false</xsl:text>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:param>
		<imsqti:feedbackType>
			<xsl:choose>
				<xsl:when test="$feedback='true' and $adaptive='true'">
					<xsl:text>adaptive</xsl:text>
				</xsl:when>
				<xsl:when test="$feedback='true' and $adaptive='false'">
					<xsl:text>nonadaptive</xsl:text>
				</xsl:when>
				<xsl:otherwise>
					<xsl:text>none</xsl:text>
				</xsl:otherwise>
			</xsl:choose>
		</imsqti:feedbackType>
	</xsl:template>
	<xsl:template name="solutionAvailable">
		<xsl:param name="clist" select="//imsqti:correctResponse"/>
		<xsl:param name="count">1</xsl:param>
		<imsqti:solutionAvailable>
			<xsl:choose>
				<xsl:when test="$count &lt;= count($clist)">
					<xsl:text>true</xsl:text>
				</xsl:when>
				<xsl:otherwise>
					<xsl:text>false</xsl:text>
				</xsl:otherwise>
			</xsl:choose>
		</imsqti:solutionAvailable>
	</xsl:template>
</xsl:stylesheet>
